@@ -46,7 +46,6 @@
 #include <sstream>
 
 
-
 #include <apt-pkg/pkgrecords.h>
 #include <apt-pkg/depcache.h>
 #include <apt-pkg/srcrecords.h>
@@ -62,6 +61,8 @@
 #include <apt-pkg/versionmatch.h>
 #include <apt-pkg/version.h>
 #include <apt-pkg/policy.h>
+#include <apt-pkg/fileutl.h>
+#include <apt-pkg/metaindex.h>
 
 #ifdef WITH_LUA
 #include <apt-pkg/luaiface.h>
@@ -69,8 +70,8 @@
 
 #include "raptoptions.h"
 
-
-static char descrBuffer[8192];
+static int descrBufferSize = 4096;
+static char *descrBuffer = new char[descrBufferSize];
 
 static char *parseDescription(string descr);
 
@@ -78,10 +79,14 @@ static char *parseDescription(string descr);
 RPackage::RPackage(RPackageLister *lister, pkgDepCache *depcache,
                    pkgRecords *records, pkgCache::PkgIterator &pkg)
 : _lister(lister), _records(records), _depcache(depcache),
-_notify(true), _boolFlags(0)
+  _notify(true), _boolFlags(0)
 {
    _package = new pkgCache::PkgIterator(pkg);
 
+#ifdef WITH_APT_MULTIARCH_SUPPORT
+   fullname = _package->FullName(true);
+#endif
+
    pkgDepCache::StateCache & State = (*_depcache)[*_package];
    if (State.CandVersion != NULL)
       _defaultCandVer = State.CandVersion;
@@ -128,7 +133,8 @@ const char *RPackage::summary()
 
    pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
    if (!ver.end()) {
-      pkgRecords::Parser & parser = _records->Lookup(ver.FileList());
+      pkgCache::DescIterator Desc = ver.TranslatedDescription();
+      pkgRecords::Parser & parser = _records->Lookup(Desc.FileList());
       _summary = parser.ShortDesc();
       return _summary.c_str();
    }
@@ -148,6 +154,18 @@ const char *RPackage::maintainer()
    return "";
 }
 
+string RPackage::arch()
+{
+   pkgCache::VerIterator ver;
+
+   ver = (*_depcache)[*_package].InstVerIter(*_depcache);
+
+   // the arch:all property is part of the version
+   if (ver && ver.Arch())
+      return ver.Arch();
+
+   return _package->Arch();
+}
 
 const char *RPackage::vendor()
 {
@@ -182,6 +200,7 @@ const char *RPackage::priority()
 const char *RPackage::installedFiles()
 {
    static string filelist;
+   vector<string> sV;
    string s;
 
    filelist.erase(filelist.begin(), filelist.end());
@@ -193,10 +212,12 @@ const char *RPackage::installedFiles()
          return "";
       while (in.eof() == false) {
          getline(in, s);
-         filelist += s + "\n";
+         sV.push_back( s );
       }
+      sort(sV.begin(), sV.end());
+      for (unsigned int i = 1; i < sV.size(); i++)
+         filelist += sV[i] + "\n";
 
-      in >> filelist;
       return filelist.c_str();
    }
    filelist = _("The list of installed files is only available for installed packages");
@@ -217,7 +238,8 @@ const char *RPackage::description()
    pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
 
    if (!ver.end()) {
-      pkgRecords::Parser & parser = _records->Lookup(ver.FileList());
+      pkgCache::DescIterator Desc = ver.TranslatedDescription();
+      pkgRecords::Parser & parser = _records->Lookup(Desc.FileList());
       _description = parseDescription(parser.LongDesc());
       return _description.c_str();
    } else {
@@ -225,6 +247,45 @@ const char *RPackage::description()
    }
 }
 
+string RPackage::getRawRecord(bool useCandidateVersion)
+{
+   pkgCache::VerIterator ver;
+   if(!useCandidateVersion)
+      ver = (*_depcache)[*_package].InstVerIter(*_depcache);
+   if(useCandidateVersion || ver.end())
+      ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
+   if(ver.end() == false) {
+      const char *start, *stop;
+      unsigned long len;
+      pkgRecords::Parser &rec=_records->Lookup(ver.FileList());
+      rec.GetRec(start, stop);
+      len = stop-start;
+      return string(start, len);
+   }
+   return string();
+}
+
+string RPackage::findTagFromPkgRecord(const char *tag)
+{
+   pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
+
+   if (!ver.end()) {
+      const char *start, *stop;
+      pkgTagSection sec;
+      unsigned long len;
+
+      pkgRecords::Parser &rec=_records->Lookup(ver.FileList());
+      rec.GetRec(start, stop);
+      len = stop-start;
+      // add +1 to ensure we have the double lineline in the buffer
+      if (start && sec.Scan(start, len+1))
+	 return sec.FindS(tag);
+   } 
+
+   return string();
+}
+
+
 long RPackage::installedSize()
 {
    pkgCache::VerIterator ver = _package->CurrentVer();
@@ -307,9 +368,43 @@ int RPackage::getFlags()
        !state.CandidateVerIter(*_depcache).Downloadable())
       flags |= FNotInstallable;
 
+   if (state.Flags & pkgCache::Flag::Auto)
+      flags |= FIsAuto;
+
+   if (state.Garbage)
+      flags |= FIsGarbage;
+
+   if (state.NowPolicyBroken())
+      flags |= FNowPolicyBroken;
+
+   if (state.InstPolicyBroken())
+      flags |= FInstPolicyBroken;
+
    return flags | _boolFlags;
 }
 
+const char* RPackage::name()
+{ 
+#ifdef WITH_APT_MULTIARCH_SUPPORT
+   return fullname.c_str();
+#else
+   const char *s = _package->Name(); 
+   if (s == NULL)
+      return "";
+   return s;
+#endif
+}
+
+bool RPackage::isMultiArchDuplicate()
+{
+   // Installed packages are never "hidden"
+   if ((*_package)->CurrentVer != 0)
+      return false;
+   // find the "best" package for the given group, if that is different
+   // from our PkgIterator this is not a interessting pkg
+   return (_package->Group().FindPkg() != *_package);
+}
+
 #if 0
 bool RPackage::isWeakDep(pkgCache::DepIterator &dep)
 {
@@ -403,9 +498,12 @@ vector<DepInformation> RPackage::enumRDeps()
 	 dep.versionComp = "";
       }
 
-      //FIXME: HACK ALARM, we need a "RDepends" type, so we use the last
-      // one in pkg-dep
-      dep.type = (pkgCache::Dep::DepType)(pkgCache::Dep::Obsoletes+1);
+      // HACK: we (ab)use the DepType for the dependency type,
+      //       but add our own RDepends type that is always the
+      //       last element of DepTypeStr[] 
+      // FIXME: make this less hacky
+      int nr_elements = sizeof(DepTypeStr)/sizeof(char*);
+      dep.type = (pkgCache::Dep::DepType)(nr_elements-1);
       dep.name = D.ParentPkg().Name();
 
       if(Trg->VersionList == 0)
@@ -686,15 +784,27 @@ bool RPackage::isTrusted()
    pkgDepCache::StateCache & State = (*_depcache)[*_package];
    Ver = State.CandidateVerIter(*_depcache);
    if (Ver == 0) {
-      cerr << "CanidateVer == 0" << endl;
+      //cerr << "CanidateVer == 0" << endl;
       return false;
    }
    pkgSourceList *Sources=_lister->getCache()->list();
+   std::map<pkgCache::PkgFileIterator, pkgIndexFile*>::iterator it;   
    for (pkgCache::VerFileIterator i = Ver.FileList(); i.end() == false; i++)
    {
       pkgIndexFile *Index;
-      if (Sources->FindIndex(i.File(),Index) == false)
-         continue;
+
+      // FIXME: instead of doing his dance (he trust cache is just
+      //        a couple of items big, that should be done in apt itself
+      it = _lister->getCache()->trust_cache().find(i.File());
+      if (it == _lister->getCache()->trust_cache().end()) {
+         if (Sources->FindIndex(i.File(),Index) == false)
+            continue;
+      } else {
+            Index = (*it).second;
+      }
+      _lister->getCache()->trust_cache().insert(
+         pair<pkgCache::PkgFileIterator, pkgIndexFile*>(i.File(), Index) );
+
       if (_config->FindB("Debug::pkgAcquire::Auth", false))
       {
          std::cerr << "Checking index: " << Index->Describe()
@@ -728,6 +838,12 @@ void RPackage::setNotify(bool flag)
    _notify = flag;
 }
 
+void RPackage::setAuto(bool flag)
+{
+   _depcache->MarkAuto(*_package, flag);
+}
+
+
 void RPackage::setKeep()
 {
    _depcache->MarkKeep(*_package, false);
@@ -790,24 +906,11 @@ void RPackage::setRemove(bool purge)
       _lister->notifyChange(this);
 }
 
-
-string RPackage::getChangelogFile(pkgAcquire *fetcher)
+string RPackage::getScreenshotFile(pkgAcquire *fetcher, bool thumb)
 {
-   string prefix;
-   string srcpkg = srcPackage();
-   string descr("Changelog for ");
+   string descr("Screenshot for ");
    descr+=name();
 
-   string src_section=section();
-   if(src_section.find('/')!=src_section.npos)
-      src_section=string(src_section, 0, src_section.find('/'));
-   else
-      src_section="main";
-
-   prefix+=srcpkg[0];
-   if(srcpkg.size()>3 && srcpkg[0]=='l' && srcpkg[1]=='i' && srcpkg[2]=='b')
-      prefix=std::string("lib")+srcpkg[3];
-
    string verstr;
    if(availableVersion() != NULL) 
       verstr = availableVersion();
@@ -815,37 +918,127 @@ string RPackage::getChangelogFile(pkgAcquire *fetcher)
    if(verstr.find(':')!=verstr.npos)
       verstr=string(verstr, verstr.find(':')+1);
    char uri[512];
-   snprintf(uri,512,"http://packages.debian.org/changelogs/pool/%s/%s/%s/%s_%s/changelog",
+   if(thumb)
+      snprintf(uri,512,"http://screenshots.debian.net/thumbnail/%s", name());
+   else
+      snprintf(uri,512,"http://screenshots.debian.net/screenshot/%s", name());
+
+   //cerr << "uri is: " << uri << endl;
+
+   string filename = RTmpDir()+"/tmp_sh";
+   unlink(filename.c_str());
+   new pkgAcqFileSane(fetcher, uri, descr, name(), filename);
+
+   int res = fetcher->Run();
+   //cerr << "res: " << res << endl;
+
+   return filename;
+}
+
+string RPackage::getChangelogURI() 
+{
+   char uri[512];
+   //FIXME: get the supportedOrigins from pkgStatus
+   if(origin() == "Debian") {
+      string prefix;
+      string srcpkg = srcPackage();
+
+      string src_section=section();
+      if(src_section.find('/')!=src_section.npos)
+         src_section=string(src_section, 0, src_section.find('/'));
+      else
+         src_section="main";
+
+      prefix+=srcpkg[0];
+      if(srcpkg.size()>3 && srcpkg[0]=='l' && srcpkg[1]=='i' && srcpkg[2]=='b')
+         prefix=std::string("lib")+srcpkg[3];
+
+      string verstr;
+      if(availableVersion() != NULL) 
+         verstr = availableVersion();
+      
+      if(verstr.find(':')!=verstr.npos)
+         verstr=string(verstr, verstr.find(':')+1);
+
+      snprintf(uri,512,"http://packages.debian.org/changelogs/pool/%s/%s/%s/%s_%s/changelog",
                                src_section.c_str(),
                                prefix.c_str(),
                                srcpkg.c_str(),
                                srcpkg.c_str(),
                                verstr.c_str());
+   } else {
+       string pkgfilename = findTagFromPkgRecord("Filename");
+       pkgfilename = pkgfilename.substr(0, pkgfilename.find_last_of('.')) + ".changelog";
+       snprintf(uri,512,"http://%s/%s",
+               getCandidateOriginSiteUrl().c_str(),
+               pkgfilename.c_str());
+   }
+   return string(uri);
+}
 
-   //cout << "uri is: " << uri << endl;
+string RPackage::getChangelogFile(pkgAcquire *fetcher)
+{
+   string descr("Changelog for ");
+   descr+=name();
 
+   string uri = getChangelogURI();
    // no need to translate this, the changelog is in english anyway
    string filename = RTmpDir()+"/tmp_cl";
-   ofstream out(filename.c_str());
-   out << "Failed to fetch the changelog for " << name() << endl;
-   out << "URI was: " << uri << endl;
-   out.close();
-   new pkgAcqFileSane(fetcher, uri, descr, name(), filename);
 
-   fetcher->Run();
+   new pkgAcqFileSane(fetcher, uri, descr, name(), filename);
+   //cerr << "**DEBUG** origin: " << origin() << endl;
+   //cerr << "**DEBUG** uri: " << uri << endl;
+   //cerr << "**DEBUG** filename: " << filename << endl;
+
+   ofstream out(filename.c_str());
+   if(fetcher->Run() == pkgAcquire::Failed) {
+      out << "Failed to download the list of changes. " << endl;
+      out << "Please check your Internet connection." << endl;
+      // FIXME: Need to dequeue the item
+   } else {
+      struct stat filestatus;
+      stat(filename.c_str(), &filestatus );
+      if (filestatus.st_size == 0) {
+         out << "This change is not coming from a source that supports changelogs.\n" << endl;
+         out << "Failed to fetch the changelog for " << name() << endl;
+         out << "URI was: " << uri << endl;
+      }
+   };
+   out.close();
 
    return filename;
 }
 
-string RPackage::getCanidateOrigin()
+string RPackage::getCandidateOriginStr()
 {
-   for(pkgCache::VerIterator Ver = _package->VersionList(); Ver.end() == false; Ver++) {
-      // we always take the first available version 
-      pkgCache::VerFileIterator VF = Ver.FileList();
-      if(!VF.end()) {
-	 return VF.File().Site();
-      }
-   }
+   pkgCache::VerIterator Ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
+   if(Ver.end())
+      return "";
+   pkgCache::VerFileIterator VF = Ver.FileList();
+   if(!VF.end() && VF.File() && VF.File().Origin())
+      return VF.File().Origin();
+   return "";
+}
+
+string RPackage::getCandidateOriginSuite()
+{
+   pkgCache::VerIterator Ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
+   if(Ver.end())
+      return "";
+   pkgCache::VerFileIterator VF = Ver.FileList();
+   if(!VF.end() && VF.File() && VF.File().Archive())
+      return VF.File().Archive();
+   return "";
+}
+
+string RPackage::getCandidateOriginSiteUrl()
+{
+   pkgCache::VerIterator Ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
+   if(Ver.end())
+      return "";
+   pkgCache::VerFileIterator VF = Ver.FileList();
+   if(!VF.end() && VF.File() && VF.File().Site())
+      return VF.File().Site();
    return "";
 }
 
@@ -886,8 +1079,10 @@ void RPackage::setPinned(bool flag)
          cerr << "error opening tmpfile: " << filename << endl;
       FileFd Fd(File, FileFd::ReadOnly);
       pkgTagFile TF(&Fd);
-      if (_error->PendingError() == true)
+      if (_error->PendingError() == true) {
+         fclose(out);
          return;
+      }
       pkgTagSection Tags;
       while (TF.Step(Tags) == true) {
          string Name = Tags.FindS("Package");
@@ -991,6 +1186,20 @@ bool RPackage::setVersion(string verTag)
 
    _depcache->SetCandidateVersion(Ver);
 
+   string archive;
+   for (pkgCache::VerFileIterator VF = Ver.FileList();
+        VF.end() == false;
+        VF++)
+   {
+      if (!VF.File() || !VF.File().Archive())
+         continue;
+      //std::cerr << "vf: " << VF.File().Archive() << std::endl;
+      archive = VF.File().Archive();
+      if(!_depcache->SetCandidateRelease(Ver, archive))
+         std::cerr << "Failed to SetCandidateRelease for " << archive << std::endl;
+      break;
+   }
+
    _boolFlags |= FOverrideVersion;
 
    return true;
@@ -1179,9 +1388,11 @@ static char *stripWsParser(string descr)
 static char *parseDescription(string descr)
 {
 
-   if (descr.size() > sizeof(descrBuffer))
-      return "Description Too Long";
-
+   if (descr.size() + 1 > descrBufferSize) {
+      delete[] descrBuffer;
+      descrBufferSize = descr.size() + 1;
+      descrBuffer = new char[descrBufferSize];
+   }
 #ifdef HAVE_RPM
    int parser = _config->FindI("Synaptic::descriptionParser", NO_PARSER);
 #else
@@ -1262,127 +1473,77 @@ string RPackage::label()
    return res;
 }
 
-
-// class that finds out what do display to get user
-void RPackageStatus::init()
+string RPackage::origin()
 {
-   char *status_short[N_STATUS_COUNT] = {
-      "install", "reinstall", "upgrade", "downgrade", "remove",
-      "purge", "available", "available-locked",
-      "installed-updated", "installed-outdated", "installed-locked",
-      "broken", "new"
-   };
-   memcpy(PackageStatusShortString, status_short, sizeof(status_short));
-
-   char *status_long[N_STATUS_COUNT] = {
-      _("Marked for installation"),
-      _("Marked for re-installation"),
-      _("Marked for upgrade"),
-      _("Marked for downgrade"),
-      _("Marked for removal"),
-      _("Marked for complete removal"),
-      _("Not installed"),
-      _("Not installed (locked)"),
-      _("Installed"),
-      _("Installed (upgradable)"),
-      _("Installed (locked to the current version)"),
-      _("Broken"),
-      _("Not installed (new in repository)")
-   };
-   memcpy(PackageStatusLongString, status_long, sizeof(status_long));
-
-
-   // check for unsupported stuff
-   if(_config->FindB("Synaptic::mark-unsupported",true)) {
-      string s, labels, components;
-      markUnsupported = true;
-
-      // read supported labels
-      labels = _config->Find("Synaptic::supported-label", "Debian Debian-Security");
-      stringstream sst1(labels);
-      while(!sst1.eof()) {
-	 sst1 >> s;
-	 supportedLabels.push_back(s);
-      }
-      
-      // read supported components
-      components = _config->Find("Synaptic::supported-components", "main updates/main");
-      stringstream sst2(components);
-      while(!sst2.eof()) {
-	 sst2 >> s;
-	 supportedComponents.push_back(s);
-      }
-   } 
-
-}
-
-bool RPackageStatus::isSupported(RPackage *pkg) 
-{
-   bool res = true;
-
-   if(markUnsupported) {
-      bool sc, sl;
-
-      sc=sl=false;
-
-      string component = pkg->component();
-      string label = pkg->label();
-
-      for(unsigned int i=0;i<supportedComponents.size();i++) {
-	 if(supportedComponents[i] == component) {
-	    sc = true;
-	    break;
-	 }
-      }
-      for(unsigned int i=0;i<supportedLabels.size();i++) {
-	 if(supportedLabels[i] == label) {
-	    sl = true;
-	    break;
-	 }
-      }
-      res = (sc & sl);
+   string res;
+   pkgCache::VerIterator Ver;
+   pkgDepCache::StateCache & State = (*_depcache)[*_package];
+   if (State.CandidateVer == 0) {
+      return "";
    }
+   Ver = State.CandidateVerIter(*_depcache);
+   pkgCache::VerFileIterator VF = Ver.FileList();
+   pkgCache::PkgFileIterator File = VF.File();
+
+   if(File.Origin() == NULL) {
+      return "";
+   }
+
+   res = File.Origin();
 
    return res;
 }
 
-int RPackageStatus::getStatus(RPackage *pkg)
+static pkgCache::PkgFileIterator
+_searchPkgFileIter(pkgCache::PkgIterator *Pkg, string label, string release)
 {
-   int flags = pkg->getFlags();
-   int ret = NotInstalled;
+   pkgCache::VerIterator Ver;
+   pkgCache::VerFileIterator VF;
+   pkgCache::PkgFileIterator PF;
 
-   if (pkg->wouldBreak()) {
-      ret = IsBroken;
-   } else if (flags & RPackage::FNewInstall) {
-      ret = ToInstall;
-   } else if (flags & RPackage::FUpgrade) {
-      ret = ToUpgrade;
-   } else if (flags & RPackage::FReInstall) {
-      ret = ToReInstall;
-   } else if (flags & RPackage::FDowngrade) {
-      ret = ToDowngrade;
-   } else if (flags & RPackage::FPurge) {
-      ret = ToPurge;
-   } else if (flags & RPackage::FRemove) {
-      ret = ToRemove;
-   } else if (flags & RPackage::FInstalled) {
-      if (flags & RPackage::FPinned)
-         ret = InstalledLocked;
-      else if (flags & RPackage::FOutdated)
-         ret = InstalledOutdated;
-      else
-         ret = InstalledUpdated;
-   } else {
-      if (flags & RPackage::FPinned)
-         ret = NotInstalledLocked;
-      else if (flags & RPackage::FNew)
-         ret = IsNew;
-      else
-         ret = NotInstalled;
+   for(Ver = Pkg->VersionList();!Ver.end();Ver++) {
+      for(VF = Ver.FileList();!VF.end(); VF++) {
+	 for(PF = VF.File(); !PF.end(); PF++) {
+	    if(!PF.end() && 
+	       PF.Label() && string(PF.Label()) == label &&
+	       PF.Origin() && string(PF.Origin()) == label &&
+	       PF.Archive() && PF.Archive() == release) {
+	       //cerr << "found: " << PF.FileName() << endl;
+	       return PF;
+	    }
+	 }
+      }
    }
-
-   return ret;
+   PF = pkgCache::PkgFileIterator(*Pkg->Cache());
+   return PF;
 }
 
+// look for a release file that matches the given label and origin string
+string RPackage::getReleaseFileForOrigin(string label, string release)
+{
+   pkgIndexFile *index;
+   pkgCache::PkgFileIterator found = _searchPkgFileIter(_package, label, string(release));
+   if (found.end())
+      return "";
+   
+   // search for the matching meta-index
+   pkgSourceList *list = _lister->getCache()->list();
+   if(list->FindIndex(found, index)) {
+      vector<metaIndex *>::const_iterator I;
+      for(I=list->begin(); I != list->end(); I++) {
+	 vector<pkgIndexFile *>  *ifv = (*I)->GetIndexFiles();
+	 if(find(ifv->begin(), ifv->end(), index) != ifv->end()) {
+	    string uri = _config->FindDir("Dir::State::lists");
+	    uri += URItoFileName((*I)->GetURI());
+	    uri += "dists_";
+	    uri += (*I)->GetDist();
+	    uri += "_Release";
+	    return uri;
+	 }
+      }
+   }
+
+   return "";
+}
 
 // vim:ts=3:sw=3:et
