@@ -22,6 +22,7 @@
  * USA
  */
 
+#include <apt-pkg/version.h>
 #include <apt-pkg/pkgrecords.h>
 #include <apt-pkg/configuration.h>
 #include <rpackage.h>
@@ -31,6 +32,7 @@
 #include <map>
 #include <vector>
 #include <sstream>
+#include <algorithm>
 
 #include "sections_trans.h"
 
@@ -64,6 +66,11 @@ void RPackageView::clear()
    _view.clear();
 }
 
+bool RPackageView::hasPackage(RPackage *pkg)
+{
+   return find(_selectedView.begin(), _selectedView.end(), pkg)  != _selectedView.end();
+}
+
 void RPackageView::clearSelection()
 {
    _hasSelection = false;
@@ -71,6 +78,19 @@ void RPackageView::clearSelection()
    _selectedView.clear();
 }
 
+void RPackageView::refresh()
+{
+   if(_config->FindB("Debug::Synaptic::View",false))
+      ioprintf(clog, "RPackageView::refresh(): '%s'\n",
+	       getName().c_str());
+
+   _view.clear();
+   for(unsigned int i=0;i<_all.size();i++) {
+      if(_all[i])
+	 addPackage(_all[i]);
+   }
+}
+
 void RPackageViewSections::addPackage(RPackage *package)
 {
    string str = trans_section(package->section());
@@ -124,8 +144,23 @@ void RPackageViewStatus::addPackage(RPackage *pkg)
    }
    _view[str].push_back(pkg);
 
+   if ((flags & RPackage::FInstalled) &&
+       (flags & RPackage::FIsGarbage))
+   {
+      str = _("Installed (auto removable)");
+      _view[str].push_back(pkg);
+   }
+
+   if ((flags & RPackage::FInstalled) &&
+       !(flags & RPackage::FIsAuto) &&
+       !(flags & RPackage::FImportant))
+   {
+      str = _("Installed (manual)");
+      _view[str].push_back(pkg);
+   }
+
    str.clear();
-   if (flags & RPackage::FNowBroken)
+   if (flags & RPackage::FNowBroken || flags & RPackage::FInstBroken)
       str = _("Broken dependencies");
    else if (flags & RPackage::FNew)
       str = _("New in repository");
@@ -147,6 +182,7 @@ void RPackageViewStatus::addPackage(RPackage *pkg)
       _view[str].push_back(pkg);
 }
 
+
 //------------------------------------------------------------------
 
 void RPackageViewSearch::addPackage(RPackage *pkg)
@@ -155,11 +191,11 @@ void RPackageViewSearch::addPackage(RPackage *pkg)
    const char *tmp=NULL;
    bool global_found=true;
 
-   if(!pkg || searchStrings.empty())
+   if(!pkg || _currentSearchItem.searchStrings.empty())
       return;
 
    // build the string
-   switch(searchType) {
+   switch(_currentSearchItem.searchType) {
    case RPatternPackageFilter::Name:
       tmp = pkg->name();
       break;
@@ -194,8 +230,8 @@ void RPackageViewSearch::addPackage(RPackage *pkg)
       str = tmp;
       
    // find the search pattern in the string "str"
-   for(unsigned int i=0;i<searchStrings.size();i++) {
-      string searchString = searchStrings[i];
+   for(unsigned int i=0;i<_currentSearchItem.searchStrings.size();i++) {
+      string searchString = _currentSearchItem.searchStrings[i];
 
       if(!str.empty() && strcasestr(str.c_str(), searchString.c_str())) {
 	 global_found &= true;
@@ -204,7 +240,7 @@ void RPackageViewSearch::addPackage(RPackage *pkg)
       }
    }
    if(global_found) {
-      _view[searchName].push_back(pkg);
+      _view[_currentSearchItem.searchName].push_back(pkg);
       found++;
    }
 
@@ -214,32 +250,75 @@ void RPackageViewSearch::addPackage(RPackage *pkg)
    //_view[searchString].push_back(NULL);
 }
 
-int RPackageViewSearch::setSearch(string aSearchName, int type, 
-				  string searchString)
+bool RPackageViewSearch::setSelected(string name)
+{
+   // if we do not have the search name in the current view, 
+   // check if we only have it in the searchHistory and if so,
+   // redo the search
+   if (_view.find(name) == _view.end()) {
+      map<string, searchItem>::iterator J = searchHistory.find(name);
+      if (J != searchHistory.end()) {
+	 //cerr << "found in search history, reapplying search" << endl;
+	 string s;
+	 OpProgress progress;
+	 for(int i=0;i < (*J).second.searchStrings.size();i++)
+	    s += string(" ") + (*J).second.searchStrings[i];
+	 // do search but do not add to history (we have it there already)
+	 setSearch((*J).second.searchName, (*J).second.searchType, s, 
+		   // FIXME: re-use progress from setSearch()
+		   progress);
+      }
+   }
+
+   // call parent
+   return RPackageView::setSelected(name);
+}
+
+vector<string> RPackageViewSearch::getSubViews()
+{
+   vector<string> subviews;
+   for(map<string, searchItem>::iterator I = searchHistory.begin();
+       I != searchHistory.end();
+       I++)
+     subviews.push_back((*I).first);
+   return subviews;
+}
+
+int RPackageViewSearch::setSearch(string aSearchName, 
+				  int type, 
+				  string searchString, 
+				  OpProgress &searchProgress)
 {
    found = 0;
-   searchType = type;
-   searchName = aSearchName;
 
-   _view[searchName].clear();
-   searchStrings.clear();
+   _currentSearchItem.searchType = type;
+   _currentSearchItem.searchName = aSearchName;
+
+   _view[_currentSearchItem.searchName].clear();
+   _currentSearchItem.searchStrings.clear();
 
    // tokenize the str and add to the searchString vector
    stringstream sstream(searchString);
    string s;
    while(!sstream.eof()) {
       sstream >> s;
-      searchStrings.push_back(s);
+      _currentSearchItem.searchStrings.push_back(s);
    }
 
-   // reapply search when a new search strng is giben
+   // overwrite existing ones
+   searchHistory[aSearchName] =  _currentSearchItem;
+   
+   // setup search progress (0 done, _all.size() in total, 1 subtask)
+   searchProgress.OverallProgress(0, _all.size(), 1, _("Searching"));
+   // reapply search when a new search strng is given
    for(unsigned int i=0;i<_all.size();i++) 
-      if(_all[i]) 
-	 addPackage(_all[i]);
-         
+     if(_all[i]) {
+       searchProgress.Progress(i);
+       addPackage(_all[i]);
+     }
+   searchProgress.Done();
    return found;
 }
-
 //------------------------------------------------------------------
 
 RPackageViewFilter::RPackageViewFilter(vector<RPackage *> &allPkgs) 
@@ -294,6 +373,14 @@ RPackageViewFilter::iterator RPackageViewFilter::begin()
    return _selectedView.begin(); 
 }
 
+void RPackageViewFilter::refresh()
+{
+   //cout << "RPackageViewFilter::refresh() " << endl;
+
+   refreshFilters();
+}
+
+
 vector<string> RPackageViewFilter::getFilterNames()
 {
    vector<string> filters;
@@ -306,9 +393,18 @@ vector<string> RPackageViewFilter::getFilterNames()
 void RPackageViewFilter::addPackage(RPackage *pkg)
 {
    // nothing to do for now, may add some sort of caching later
-   _sectionList.insert(pkg->section());
 }
 
+const set<string>& RPackageViewFilter::getSections() 
+{ 
+   if(_sectionList.empty())
+      for(unsigned int i=0;i<_all.size();i++)
+	 if(_all[i])
+	    _sectionList.insert(_all[i]->section());
+   return _sectionList; 
+};
+
+
 void RPackageViewFilter::storeFilters()
 {
    ofstream out;
@@ -451,7 +547,94 @@ void RPackageViewFilter::makePresetFilters()
    filter->status.setStatus(RStatusPackageFilter::UpstreamUpgradable);
    filter->setName("Upgradable (upstream)"); _("Upgradable (upstream)");
    registerFilter(filter);
+
+   filter = new RFilter();
+   filter->preset = true;
+   filter->pattern.addPattern(RPatternPackageFilter::Component,
+			      "main", true);
+   filter->pattern.addPattern(RPatternPackageFilter::Component,
+			      "restricted", true);
+   filter->pattern.addPattern(RPatternPackageFilter::Origin,
+			      "Ubuntu", false);
+   filter->pattern.setAndMode(true);
+   filter->status.setStatus(RStatusPackageFilter::Installed);
+   filter->setName("Community Maintained (installed)"); _("Community Maintained (installed)");
+   registerFilter(filter);
+
+   filter = new RFilter();
+   filter->preset = true;
+   filter->status.setStatus(RStatusPackageFilter::NowPolicyBroken);
+   filter->setName("Missing Recommends"); _("Missing Recommends");
+   registerFilter(filter);
 }
 
+void RPackageViewOrigin::addPackage(RPackage *package)
+{
+   string subview;
+   string component =  package->component();
+   string origin_url = package->getCandidateOriginSiteUrl();
+   string suite  = package->getCandidateOriginSuite();
+   string origin_str  = package->getCandidateOriginStr();
+
+   // local origins are all put under local (no matter what component, section)
+   if(origin_url == "") {
+      origin_url = _("Local");
+      _view[origin_url].push_back(package);
+      return;
+   }
+
+   // PPAs are special too
+   if(origin_str.find("LP-PPA-") != string::npos) {
+     _view[origin_str+"/"+suite].push_back(package);
+     return;
+   }
+
+   if(component == "")
+      component = _("Unknown");
+   if(suite == "now")
+      suite = "";
+
+   // normal package
+   subview = suite+"/"+component+" ("+origin_url+")";
+   _view[subview].push_back(package);
+
+   // see if we have versions that are higher than the candidate
+   // (e.g. experimental/backports)
+   for (pkgCache::VerIterator Ver = package->package()->VersionList();
+        Ver.end() == false; Ver++)
+   {
+      pkgCache::VerFileIterator VF = Ver.FileList();
+      if ( (VF.end() == true) || (VF.File() == NULL) )
+         continue;
+      // ignore versions that are lower or equal than the candidate
+      if (_system->VS->CmpVersion(Ver.VerStr(), 
+                                  package->availableVersion()) <= 0)
+         continue;
+      // ignore "now"
+      if(strcmp(VF.File().Archive(), "now") == 0)
+         continue;
+      //std::cerr << "version.second: " << version.second 
+      //          << " origin_str: " << suite << std::endl;
+      string prefix = _("Not automatic: ");
+      string suite = VF.File().Archive();
+      string origin_url = VF.File().Site();
+      string subview = prefix + suite + "(" + origin_url + ")";
+      _view[subview].push_back(package);
+   }
+ };
+
+
+void RPackageViewArchitecture::addPackage(RPackage *package)
+{
+   string arch = "arch: " + package->arch();
+
+   // FIXME: add pseudo arch for packages only in one group arch
+   //        but not the other
+   
+
+   _view[arch].push_back(package);
+};
+
+
 
 // vim:sts=3:sw=3
